﻿//   TntWare CS OpenID Provider
//      Copyright (c) 2008, TntWare (http://www.tntware.com), Campus Crusade for Christ
//         By Troy Wolbrink, troy.wolbrink@ccci.org 

using CommunityServer.Components;
using DotNetOpenId;
using DotNetOpenId.RelyingParty;
using DotNetOpenId.Extensions.SimpleRegistration;
using DotNetOpenId.Extensions.ProviderAuthenticationPolicy;
using System;
using System.Web.UI;

namespace TntWare.CsOpenId
{

    public class MyOpenIdTextBox : OpenIdTextBox
    {
        Identifier MyOpenID = null;

        public MyOpenIdTextBox()
        {
            this.LoggedIn += new EventHandler<DotNetOpenId.RelyingParty.OpenIdEventArgs>(this.MyLoggedIn);
            this.Failed += new EventHandler<DotNetOpenId.RelyingParty.OpenIdEventArgs>(this.MyFailed);
            this.SetupRequired += new EventHandler<DotNetOpenId.RelyingParty.OpenIdEventArgs>(this.MySetupRequired);
            this.Canceled += new EventHandler<DotNetOpenId.RelyingParty.OpenIdEventArgs>(this.MyCanceled);
        }

        internal IAuthenticationRequest GetRequest()
        {
            if (Request == null)
            {
                CreateRequest();
            }
            return Request;
        }

        internal Identifier CompleteRequest()
        {
            MyOpenID = null;
            this.OnLoad(new EventArgs());
            return MyOpenID;
        }

        private void MyLoggedIn(object sender, DotNetOpenId.RelyingParty.OpenIdEventArgs e)
        {
            MyOpenID = e.ClaimedIdentifier;
            e.Cancel = true;
        }

        private void MyCanceled(object sender, DotNetOpenId.RelyingParty.OpenIdEventArgs e)
        {
            MyOpenID = null;
            if (e.Response.Exception == null)
                throw new Exception("OpenID authentication cancelled.");
            else
                throw e.Response.Exception;
        }

        private void MyFailed(object sender, DotNetOpenId.RelyingParty.OpenIdEventArgs e)
        {
            MyOpenID = null;
            if (e.Response.Exception == null)
                throw new Exception("OpenID authentication failed.");
            else
                throw e.Response.Exception;
        }

        private void MySetupRequired(object sender, DotNetOpenId.RelyingParty.OpenIdEventArgs e)
        {
            MyOpenID = null;
            if (e.Response.Exception == null)
                throw new Exception("OpenID immediate authentication requires setup.");
            else
                throw e.Response.Exception;
        }

        private DemandLevel GetDemandLevel(string regfield, string[] requiredFields, string[] requestFields)
        {
            foreach (string r in requiredFields)
            {
                if (r.ToLower() == regfield)
                {
                    return DemandLevel.Require;
                }
            }
            foreach (string r in requestFields)
            {
                if (r.ToLower() == regfield)
                {
                    return DemandLevel.Request;
                }
            }
            return DemandLevel.NoRequest;
        }

        internal void SetRequestedFields(string[] requiredFields, string[] requestFields)
        {
            this.RequestNickname = GetDemandLevel("nickname", requiredFields, requestFields);
            this.RequestEmail = GetDemandLevel("email", requiredFields, requestFields);
            this.RequestFullName = GetDemandLevel("fullname", requiredFields, requestFields);
            this.RequestBirthDate = GetDemandLevel("dob", requiredFields, requestFields);
            this.RequestGender = GetDemandLevel("gender", requiredFields, requestFields);
            this.RequestPostalCode = GetDemandLevel("postcode", requiredFields, requestFields);
            this.RequestCountry = GetDemandLevel("country", requiredFields, requestFields);
            this.RequestLanguage = GetDemandLevel("language", requiredFields, requestFields);
            this.RequestTimeZone = GetDemandLevel("timezone", requiredFields, requestFields);
        }
    }

    public class DotNetOpenIdProvider : OpenIdProvider
    {
        public override Uri CompleteConsumption(Page page)
        {
            using (MyOpenIdTextBox OpenID = new MyOpenIdTextBox())
            {
                OpenID.Page = page;
                return new Uri(OpenID.CompleteRequest());
            }
        }

        public override Uri NormalizeUri(string uriStr)
        {
            return new Uri((DotNetOpenId.Identifier)uriStr);
        }

        public const string ImmediateExt = "immediate.mode.tntware.com";
        public const string RenewExt = "renew.mode.tntware.com";

        public override void PerformLogin(string openId, Page page, string[] requiredFields, string[] requestFields)
        {
            if (string.IsNullOrEmpty(openId))
            {
                throw new ArgumentException("openId argument not provided.");
            }
            bool ImmediateMode = openId.EndsWith(ImmediateExt);
            if (ImmediateMode)
            {
                openId = openId.Substring(0, openId.Length - ImmediateExt.Length);
            }
            bool RenewMode = openId.EndsWith(RenewExt);
            if (RenewMode)
            {
                openId = openId.Substring(0, openId.Length - RenewExt.Length);
            }
            using (MyOpenIdTextBox OpenID = new MyOpenIdTextBox())
            {
                OpenID.Page = page;
                OpenID.Text = openId;
                OpenID.ImmediateMode = ImmediateMode;
                OpenID.SetRequestedFields(requiredFields, requestFields);
                if (RenewMode)
                {
                    PolicyRequest max_auth_policy = new PolicyRequest();
                    max_auth_policy.MaximumAuthenticationAge = new TimeSpan(0, 0, 1); // 1 second
                    OpenID.GetRequest().AddExtension(max_auth_policy);
                }
                try
                {
                    OpenID.LogOn();
                }
                catch (System.Threading.ThreadAbortException)
                {
                    // expected
                }
            }
        }
    }

}

